@@ -11,11 +11,13 @@ import copy
 import os
 import base64
 import time
+import fnmatch
 from struct import pack, unpack
 from datetime import timedelta
 from stat import ST_MODE
 # from lib389.utils import print_nice_time
 from lib389.paths import Paths
+from lib389._mapped_object_lint import DSLint
 from lib389.lint import (
     DSPERMLE0001,
     DSPERMLE0002,
@@ -25,21 +27,24 @@ from lib389.lint import (
 )
 
 
-class DSEldif(object):
+class DSEldif(DSLint):
     """A class for working with dse.ldif file
 
     :param instance: An instance
     :type instance: lib389.DirSrv
     """
 
-    def __init__(self, instance, serverid=None):
+    def __init__(self, instance, serverid=None, path=None):
         self._instance = instance
         self._contents = []
 
-        if serverid:
+        if path:
+            self.path = path
+        elif serverid:
             # Get the dse.ldif from the instance name
             prefix = os.environ.get('PREFIX', ""),
-            serverid = serverid.replace("slapd-", "")
+            if serverid.startswith("slapd-"):
+                serverid = serverid.replace("slapd-", "", 1)
             self.path = "{}/etc/dirsrv/slapd-{}/dse.ldif".format(prefix[0], serverid)
         else:
             ds_paths = Paths(self._instance.serverid, self._instance)
@@ -51,22 +56,16 @@ class DSEldif(object):
                 if not line.startswith(' '):
                     if processed_line:
                         self._contents.append(processed_line)
-
                     if line.startswith('dn:'):
                         processed_line = line.lower()
                     else:
                         processed_line = line
                 else:
                     processed_line = processed_line[:-1] + line[1:]
-        self._lint_functions = [self._lint_nsstate]
 
-    def lint(self):
-        results = []
-        for fn in self._lint_functions:
-            for result in fn():
-                if result is not None:
-                    results.append(result)
-        return results
+    @classmethod
+    def lint_uid(cls):
+        return 'dseldif'
 
     def _lint_nsstate(self):
         suffixes = self.readNsState()
@@ -88,6 +87,7 @@ class DSEldif(object):
                 report['items'].append('Time Skew')
                 report['items'].append('Skew: ' + suffix['time_skew_str'])
                 report['fix'] = report['fix'].replace('YOUR_INSTANCE', self._instance.serverid)
+                report['check'] = f'dseldif:nsstate'
                 yield report
 
     def _update(self):
@@ -96,6 +96,11 @@ class DSEldif(object):
         with open(self.path, "w") as file_dse:
             file_dse.write("".join(self._contents))
 
+    def globalSubstitute(self, strfrom, strto):
+        for i in range(0, len(self._contents)-1):
+            self._contents[i] = self._contents[i].replace(strfrom, strto)
+        self._update()
+
     def _find_attr(self, entry_dn, attr):
         """Find all attribute values and indexes under a given entry
 
@@ -147,6 +152,34 @@ class DSEldif(object):
             return vals[0] if len(vals) > 0 else None
         return vals
 
+    def get_indexes(self, backend):
+        """Return a list of backend indexes
+
+        :param backend: a backend to get the indexes of
+        """
+        indexes = []
+        for entry in self._contents:
+            if fnmatch.fnmatch(entry, "*,cn=index,cn={}*".format(backend.lower())):
+                start = entry.find("cn=")
+                end = entry.find(",")
+                indexes.append(entry[start+len('cn='):end])
+
+        return indexes
+
+
+    def add_entry(self, entry):
+        """Add a new entry
+
+        :param entry: the entry to add in ldif format
+        :type value: str list
+        """
+
+        if self._contents[-1] != "\n":
+            self._contents.append("\n")
+        self._contents.extend(entry)
+        self._update()
+
+
     def add(self, entry_dn, attr, value):
         """Add an attribute under a given entry
 
@@ -162,6 +195,54 @@ class DSEldif(object):
         self._contents.insert(entry_dn_i+1, "{}: {}\n".format(attr, value))
         self._update()
 
+    def rename(self, entry_dn, new_dn, del_old_rdn=True):
+        """Rename an entry
+        :param entry_dn: a DN of entry we want to rename
+        :type entry_dn: str
+        :param new_dn: The new DN
+        :type new_dn: str
+        :param del_old_rdn: Delete the old RND attribute from the entry
+        :type del_old_rdn: boolean
+        """
+
+        # old rdn
+        old_rdn = entry_dn.split(',', 1)[0]
+        rdn_attr = old_rdn.split('=')[0]
+
+        # new rdn
+        new_rdn = new_dn.split(',', 1)[0]
+        new_rdn_attr = new_rdn.split('=')[0]
+        new_rdn_val = new_rdn.split('=')[1]
+
+        # Handle the rdn attribute
+        if del_old_rdn:
+           self.delete(entry_dn, rdn_attr)
+        self.add(entry_dn, new_rdn_attr, new_rdn_val)
+
+        # Rename the entry
+        entry_dn_i = self._contents.index("dn: {}\n".format(entry_dn.lower()))
+        self._contents[entry_dn_i] = f"dn: {new_dn}\n"
+        self._update()
+
+    def delete_dn(self, entry_dn):
+        """Delete the whole entry by DN
+
+        :param entry_dn: a DN of an entry we want to delete
+        :type entry_dn: str
+        """
+
+        entry_dn_i = self._contents.index("dn: {}\n".format(entry_dn.lower()))
+
+        # Find where the entry ends
+        try:
+            dn_end_i = self._contents[entry_dn_i:].index("\n")
+        except ValueError:
+            # We are in the end of the list
+            dn_end_i = len(self._contents)
+
+        del self._contents[entry_dn_i:entry_dn_i+dn_end_i+1]
+        self._update()
+
     def delete(self, entry_dn, attr, value=None):
         """Delete attributes under a given entry
 
@@ -319,8 +400,45 @@ class DSEldif(object):
 
         return states
 
+    def _increaseTimeSkew(self, suffix, timeSkew):
+        # Increase csngen state local_offset by timeSkew
+        # Warning: instance must be stopped before calling this function
+        assert (timeSkew >= 0)
+        nsState = self.readNsState(suffix)[0]
+        self._instance.log.debug(f'_increaseTimeSkew nsState is {nsState}')
+        oldNsState = self.get(nsState['dn'], 'nsState', True)
+        self._instance.log.debug(f'oldNsState is {oldNsState}')
 
-class FSChecks(object):
+        # Lets reencode the new nsState
+        from lib389.utils import print_nice_time
+        if pack('<h', 1) == pack('=h',1):
+            end = '<'
+        elif pack('>h', 1) == pack('=h',1):
+            end = '>'
+        else:
+            raise ValueError("Unknown endian, unable to proceed")
+
+        thelen = len(oldNsState)
+        if thelen <= 20:
+            pad = 2 # padding for short H values
+            timefmt = 'I' # timevals are unsigned 32-bit int
+        else:
+            pad = 6 # padding for short H values
+            timefmt = 'Q' # timevals are unsigned 64-bit int
+        fmtstr = "%sH%dx3%sH%dx" % (end, pad, timefmt, pad)
+        newNsState = base64.b64encode(pack(fmtstr, int(nsState['rid']),
+           int(nsState['gen_time']), int(nsState['local_offset'])+timeSkew,
+           int(nsState['remote_offset']), int(nsState['seq_num'])))
+        newNsState = newNsState.decode('utf-8')
+        self._instance.log.debug(f'newNsState is {newNsState}')
+        # Lets replace the value.
+        (entry_dn_i, attr_data) = self._find_attr(nsState['dn'], 'nsState')
+        attr_i = next(iter(attr_data))
+        self._contents[entry_dn_i + attr_i] = f"nsState:: {newNsState}"
+        self._update()
+
+
+class FSChecks(DSLint):
     """This is for the healthcheck feature, check commonly used system config files the
     server uses.  This is here for lack of a better place to add this class.
     """
@@ -344,17 +462,10 @@ class FSChecks(object):
                 'report': DSPERMLE0002
             },
         ]
-        self._lint_functions = [self._lint_file_perms]
 
-    def lint(self):
-        """Run a lint/healthcheck for this class
-        """
-        results = []
-        for fn in self._lint_functions:
-            for result in fn():
-                if result is not None:
-                    results.append(result)
-        return results
+    @classmethod
+    def lint_uid(cls):
+        return 'fschecks'
 
     def _lint_file_perms(self):
         """Test file permissions are safe
@@ -370,6 +481,7 @@ class FSChecks(object):
                     report['detail'] = report['detail'].replace('PERMS', perms)
                     report['fix'] = report['fix'].replace('FILE', ds_file['name'])
                     report['fix'] = report['fix'].replace('PERMS', perms)
+                    report['check'] = f'fschecks:file_perms'
                     yield report
             except FileNotFoundError:
                 pass
