@@ -1,5 +1,5 @@
 # --- BEGIN COPYRIGHT BLOCK ---
-# Copyright (C) 2015 Red Hat, Inc.
+# Copyright (C) 2022 Red Hat, Inc.
 # Copyright (C) 2019 William Brown <william@blackhats.net.au>
 # All rights reserved.
 #
@@ -24,6 +24,7 @@ except ImportError:
         p.stdout, p.stdin = popen2(cmd_l)
         return p
 
+import json
 import re
 import os
 import logging
@@ -32,7 +33,7 @@ import ldap
 import socket
 import time
 import stat
-from datetime import datetime
+from datetime import (datetime, timedelta)
 import sys
 import filecmp
 import pwd
@@ -51,11 +52,11 @@ from ldapurl import LDAPUrl
 from contextlib import closing
 
 import lib389
-from lib389.paths import Paths
+from lib389.paths import ( Paths, DEFAULTS_PATH )
 from lib389.dseldif import DSEldif
 from lib389._constants import (
         DEFAULT_USER, VALGRIND_WRAPPER, DN_CONFIG, CFGSUFFIX, LOCALHOST,
-        ReplicaRole, CONSUMER_REPLICAID, SENSITIVE_ATTRS
+        ReplicaRole, CONSUMER_REPLICAID, SENSITIVE_ATTRS, DEFAULT_DB_LIB
     )
 from lib389.properties import (
         SER_HOST, SER_USER_ID, SER_GROUP_ID, SER_STRICT_HOSTNAME_CHECKING, SER_PORT,
@@ -195,11 +196,15 @@ def selinux_present():
     try:
         import selinux
         if selinux.is_selinux_enabled():
-            # We have selinux, continue.
-            status = True
+            # We have selinux, lets see if we are allowed to configure it.
+            # (just checking the uid for now - we may rather check if semanage command success)
+            if os.geteuid() != 0:
+                log.info('Non privileged user cannot use semanage, will not relabel ports or files.' )
+            else:
+                status = True
         else:
             # We have the module, but it's disabled.
-            log.error('selinux is disabled, will not relabel ports or files.' )
+            log.info('selinux is disabled, will not relabel ports or files.' )
     except ImportError:
         # No python module, so who knows what state we are in.
         log.error('selinux python module not found, will not relabel files.' )
@@ -265,13 +270,15 @@ def selinux_label_port(port, remove_label=False):
     :type remove_label: boolean
     :raises: ValueError: Error message
     """
+    if port is None:
+        return
     try:
         import selinux
     except ImportError:
         log.debug('selinux python module not found, skipping port labeling.')
         return
 
-    if not selinux.is_selinux_enabled():
+    if not selinux_present():
         log.debug('selinux is disabled, skipping port relabel')
         return
 
@@ -488,8 +495,10 @@ def valgrind_enable(sbin_dir, wrapper=None):
     :raise EnvironmentError: If script is not run as 'root'
     '''
 
-    if os.geteuid() != 0:
-        log.error('This script must be run as root to use valgrind')
+    if not os.access(sbin_dir, os.W_OK):
+        # Note: valgrind has no limitation but  ns-slapd must be replaced
+        # This check allows non root user to use custom install prefix
+        log.error('This script must be run as root to use valgrind (Should at least be able to write in {sbin_dir})')
         raise EnvironmentError
 
     if not wrapper:
@@ -535,7 +544,20 @@ def valgrind_enable(sbin_dir, wrapper=None):
                       e.strerror)
 
     # Disable selinux
-    os.system('setenforce 0')
+    if os.geteuid() == 0:
+        os.system('setenforce 0')
+
+    # Disable systemd by turning off with_system in .inf file
+    old_path = Paths()._get_defaults_loc(DEFAULTS_PATH)
+    new_path = f'{old_path}.orig'
+    os.rename(old_path, new_path)
+    with open(new_path, 'rt') as fin:
+       with open(old_path, 'wt') as fout:
+            for line in fin:
+                if line.startswith('with_systemd'):
+                    fout.write('with_systemd = 0\n')
+                else:
+                    fout.write(line)
 
     log.info('Valgrind is now enabled.')
 
@@ -552,8 +574,10 @@ def valgrind_disable(sbin_dir):
     :raise EnvironmentError: If script is not run as 'root'
     '''
 
-    if os.geteuid() != 0:
-        log.error('This script must be run as root to use valgrind')
+    if not os.access(sbin_dir, os.W_OK):
+        # Note: valgrind has no limitation but  ns-slapd must be replaced
+        # This check allows non root user to use custom install prefix
+        log.error('This script must be run as root to use valgrind (Should at least be able to write in {sbin_dir})')
         raise EnvironmentError
 
     nsslapd_orig = '%s/ns-slapd' % sbin_dir
@@ -577,7 +601,14 @@ def valgrind_disable(sbin_dir):
                          e.strerror)
 
     # Enable selinux
-    os.system('setenforce 1')
+    if os.geteuid() == 0:
+        os.system('setenforce 1')
+
+    # Restore .inf file (for systemd)
+    new_path = Paths()._get_defaults_loc(DEFAULTS_PATH)
+    old_path = f'{new_path}.orig'
+    if os.path.exists(old_path):
+        os.replace(old_path, new_path)
 
     log.info('Valgrind is now disabled.')
 
@@ -603,7 +634,7 @@ def valgrind_get_results_file(dirsrv_inst):
 
     # Run the command and grab the output
     p = os.popen(cmd)
-    results_file = p.readline()
+    results_file = p.readline().strip()
     p.close()
 
     return results_file
@@ -661,7 +692,8 @@ def isLocalHost(host_name):
         Uses gethostbyname()
     """
     # first see if this is a "well known" local hostname
-    if host_name == 'localhost' or \
+    if host_name is None or \
+       host_name == 'localhost' or \
        host_name == 'localhost.localdomain' or \
        host_name == socket.gethostname():
         return True
@@ -1021,25 +1053,25 @@ def formatInfData(args):
 
 def generate_ds_params(inst_num, role=ReplicaRole.STANDALONE):
     """Generate a host, port, secure port, server ID and replica ID
-    for the selected role and instance number. I.e. inst_num=1, role="master".
+    for the selected role and instance number. I.e. inst_num=1, role="supplier".
 
     @param inst_num - an instance number in a range from 1 to 99
-    @param role - ReplicaRole.STANDALONE, ReplicaRole.MASTER, ReplicaRole.HUB, ReplicaRole.CONSUMER
+    @param role - ReplicaRole.STANDALONE, ReplicaRole.SUPPLIER, ReplicaRole.HUB, ReplicaRole.CONSUMER
     @return - the dict with next keys: host, port, secure port, server id and replica id
     """
 
     if inst_num not in range(1, 100):
         raise ValueError("Instance number should be in a range from 1 to 99")
 
-    if role not in (ReplicaRole.STANDALONE, ReplicaRole.MASTER, ReplicaRole.HUB, ReplicaRole.CONSUMER):
-        raise ValueError('Role should be {}, {}, {}, {}'.format(ReplicaRole.STANDALONE, ReplicaRole.MASTER,
+    if role not in (ReplicaRole.STANDALONE, ReplicaRole.SUPPLIER, ReplicaRole.HUB, ReplicaRole.CONSUMER):
+        raise ValueError('Role should be {}, {}, {}, {}'.format(ReplicaRole.STANDALONE, ReplicaRole.SUPPLIER,
                                                                 ReplicaRole.HUB, ReplicaRole.CONSUMER))
 
     instance_data = {}
     relevant_num = 38900
 
     # Set relevant number for ports
-    if role == ReplicaRole.MASTER:
+    if role == ReplicaRole.SUPPLIER:
         relevant_num += 100
     elif role == ReplicaRole.HUB:
         relevant_num += 200
@@ -1047,7 +1079,7 @@ def generate_ds_params(inst_num, role=ReplicaRole.STANDALONE):
         relevant_num += 300
 
     # Define replica ID
-    if role == ReplicaRole.MASTER:
+    if role == ReplicaRole.SUPPLIER:
         replica_id = inst_num
     else:
         replica_id = CONSUMER_REPLICAID
@@ -1300,6 +1332,15 @@ def get_instance_list():
     insts.sort()
     return insts
 
+# used by arg parse completers
+def instance_choices(prefix, parsed_args, **kwargs):
+    insts = get_instance_list()
+    inst_names = []
+    for inst in insts:
+        inst_names.append(inst)
+        inst_names.append(inst.replace("slapd-", ""))
+
+    return inst_names
 
 def get_user_is_ds_owner():
     # Check if we have permission to administer the DS instance. This is required
@@ -1418,3 +1459,170 @@ def cmp(self, x, y):
     and strictly positive if x > y.
     """
     return (x > y) - (x < y)
+
+
+def is_valid_hostname(hostname):
+    if len(hostname) > 255:
+        return False
+    if hostname[-1] == ".":
+        hostname = hostname[:-1] # strip exactly one dot from the right, if present
+    allowed = re.compile("(?!-)[A-Z\d-]{1,63}(?<!-)$", re.IGNORECASE)
+    return all(allowed.match(x) for x in hostname.split("."))
+
+def get_default_db_lib():
+    return os.getenv('NSSLAPD_DB_LIB', default=DEFAULT_DB_LIB)
+
+
+def is_fips():
+    if os.path.exists('/proc/sys/crypto/fips_enabled'):
+        with open('/proc/sys/crypto/fips_enabled', 'r') as f:
+            state = f.readline().strip()
+            if state == '1':
+                return True
+            else:
+                return False
+    else:
+        return False
+
+
+def convert_timestamp(timestamp):
+    """Convert createtimestamp to ctime: 20170405184656Z ----> Wed Apr  5 19:46:56 2017
+    :param timestamp - A timestamp from the server
+    :return - the ctime of the timestamp
+    """
+    if timestamp:
+        time_tuple = (int(timestamp[:4]), int(timestamp[4:6]), int(timestamp[6:8]),
+                      int(timestamp[8:10]), int(timestamp[10:12]), int(timestamp[12:14]),
+                      0, 0, 0)
+        secs = time.mktime(time_tuple)
+        return time.ctime(secs)
+    else:
+        return ""
+
+
+def elapsed_time(timestamp1, timestamp2):
+    """Take two timestamps from the server and get the diff
+    """
+    if timestamp1 and timestamp2:
+        time1_tuple = (int(timestamp1[:4]), int(timestamp1[4:6]), int(timestamp1[6:8]),
+                       int(timestamp1[8:10]), int(timestamp1[10:12]), int(timestamp1[12:14]),
+                       0, 0, 0)
+        time2_tuple = (int(timestamp2[:4]), int(timestamp2[4:6]), int(timestamp2[6:8]),
+                       int(timestamp2[8:10]), int(timestamp2[10:12]), int(timestamp2[12:14]),
+                       0, 0, 0)
+        time1_secs = time.mktime(time1_tuple)
+        time2_secs = time.mktime(time2_tuple)
+        elapsed_secs = time2_secs - time1_secs
+        return str(timedelta(seconds=elapsed_secs))
+    else:
+        return ""
+
+def get_task_status(inst, log, taskObj, dn=None, show_log=False, watch=False, use_json=False):
+    """Get the status of a Task, if "watching" keep looping over the tasks
+    :param inst: DirSrv Object
+    :type inst: Object
+    :param log: Logging Object
+    :type log: Object
+    :param taskObj: Lib389 Object (task object from plugins.py)
+    :type taskObj: Object
+    :param dn: A task DN to get the status for (if omitted all tasks for checked)
+    :type dn: str
+    :param show_log: Show the task's log attribute (could be lengthy)
+    :type show_log: boolean
+    :param watch: Keep looping over the task until it completes
+    :type watch: boolean
+    :param use_json: Report the status in JSON
+    :type use_json: boolean
+    """
+
+    tasks = taskObj(inst).list()
+    all_finished = False
+    while not all_finished:
+        task_list = []
+        all_finished = True
+        for task in tasks:
+            task_base_dn = task.get_attr_val_utf8_l('basedn')
+            if task_base_dn is None:
+                # Try linked attr, it uses a different attr name
+                task_base_dn = task.get_attr_val_utf8_l('linkdn')
+            task_filter = task.get_attr_val_utf8_l('filter')
+            status = task.get_attr_val_utf8('nsTaskStatus')
+            exitcode = task.get_attr_val_utf8('nsTaskExitcode')
+            task_log =  task.get_attr_val_utf8('nsTaskLog')
+            task_created = task.get_attr_val_utf8('nsTaskCreated')
+            task_ended = task.get_attr_val_utf8('modifyTimestamp')
+            if task_base_dn is None:
+                task_base_dn = ""
+            if task_filter is None:
+                task_filter = ""
+            if status is None:
+                status = ""
+            if exitcode is None:
+                all_finished = False
+                task_ended = ""
+                exitcode = "Not finished ..."
+
+                # Calc elapsed time for running task
+                curr_time = int(time.time())  # Use the current time
+                time_tuple = (int(task_created[:4]), int(task_created[4:6]), int(task_created[6:8]),
+                              int(task_created[8:10]), int(task_created[10:12]), int(task_created[12:14]),
+                              0, 0, 0)
+                start_time = int(time.mktime(time_tuple))
+                elapsed_secs = curr_time - start_time
+                elapsed_time_str = str(timedelta(seconds=elapsed_secs))
+            else:
+                # Task is finished, use start and end times to calc elapsed time
+                elapsed_time_str = elapsed_time(task_created, task_ended)
+
+            if not show_log:
+                task_log = ""
+
+            if (dn is not None and dn.lower() == task.dn.lower()) or dn is None:
+                    task_list.append({
+                        'dn': task.dn,
+                        'basedn': task_base_dn,
+                        'filter': task_filter,
+                        'log': task_log,
+                        'status': status,
+                        'created': task_created,
+                        'created_str': convert_timestamp(task_created),
+                        'ended': task_ended,
+                        'ended_str': convert_timestamp(task_ended),
+                        'elapsed_time':  elapsed_time_str,
+                        'exitcode': exitcode
+                    })
+
+        # Display the status
+        if use_json:
+            log.info(json.dumps({"type": "list", "items": task_list}, indent=4))
+        else:
+            num_of_tasks = len(task_list)
+            count = 1
+            plural = ""
+            if num_of_tasks != 1:
+                plural = "s"
+            log.info(f'Found {num_of_tasks} fix-up task{plural}\n')
+
+            for task in task_list:
+                log.info(f"[{count}] Task: {task['dn']}")
+                log.info("-" * 80)
+                log.info(f" - Base DN:       {task['basedn']}")
+                if task['filter']:
+                    log.info(f" - Filter:        {task['filter']}")
+                log.info(f" - Status:        {task['status']}")
+                log.info(f" - Started:       {task['created_str']} ({task['created']})")
+                if task['ended_str']:
+                    log.info(f" - Ended:         {task['ended_str']} ({task['ended']})")
+                else:
+                    log.info(" - Ended:")
+                log.info(f" - Elapsed Time:  {task['elapsed_time']}")
+                log.info(f" - Exit Code:     {task['exitcode']}")
+                if show_log:
+                    log.info(f" - Log:\n{task['log']}")
+                log.info("")
+                count += 1
+        if not watch or all_finished:
+            break
+        elif not use_json:
+            log.info("Refreshing status ...")
+        time.sleep(2)
